/*! @addtogroup MCG Multipurpose Clock Generator
 *  @{
 *  @ingroup 	DEVICE_DRIVERS
 * 	@file		mcg.h
 *  @brief 		MCG Header File.
 *  @}
 */
#ifndef __MCG_H__
#define __MCG_H__

#include "derivative.h"

/*!MCG Mode error codes*/
typedef enum{
	FEI_ERROR	= 0x01, /**<Not in FEI mode*/
	FEE_ERROR	= 0x02, /**<Not in FEE mode*/
	FBI_ERROR	= 0x03, /**<Not in FBI mode*/
	FBE_ERROR	= 0x04, /**<Not in FBE mode*/
	BLPI_ERROR	= 0x05, /**<Not in BLPI mode*/
	BLPE_ERROR	= 0x06, /**<Not in BLPE mode*/
	PBE_ERROR	= 0x07, /**<Not in PBE mode*/
	PEE_ERROR	= 0x08  /**<Not in PEE mode*/
}MCG_MODE_ERROR;

/*!Clock MUX switching error codes*/
typedef enum {
	IREFST_NOT_CLEAR 	= 0x11 /**<IREFST did not clear within allowed time, FLL reference did not switch over from internal to external clock*/,
	IREFST_NOT_SET		= 0x12 /**<IREFST did not set within allowed time, the FLL reference did not switch over from external to internal clock(NEED TO CHECK IN MOVES TO FBI MODE)*/,
	IRCST_NOT_CLEAR		= 0x13 /**<IRCST did not clear within allowed time, slow IRC is not selected*/,
	IRCST_NOT_SET		= 0x14 /**<IRCST did not set within allowed time, fast IRC is not selected*/,
	PLLST_NOT_CLEAR		= 0x15 /**<PLLST did not clear, PLLST did not switch to FLL output, FLL is not running*/,
	PLLST_NOT_SET		= 0x16 /**<PLLST did not set, PLLST did not switch to PLL ouptut, PLL is not running*/,
	PLLCST_NOT_OK		= 0x17 /**<PLLCST did not switch to the correct state, the correct PLL is not selected as PLLS clock source*/,
	CLKST_NOT_0			= 0x18 /**<CLKST != 0, MCG did not switch to FLL output*/,
	CLKST_NOT_1			= 0x19 /**<CLKST != 1, MCG did not switch to internal reference clock source*/,
	CLKST_NOT_2			= 0x1A /**<CLKST != 2, MCG did not switch to external clock source*/,
	CLKST_NOT_3			= 0x1B /**<CLKST != 3, MCG did not switch to PLL*/
}MUX_SW_ERROR;

/*!Oscillator error codes*/
typedef enum{
	EXT_REF_OOR		= 0x21, /**<External frequency is greater than the maximum frequency*/
	XTAL_FREQ_OOR	= 0x22, /**<Crystal frequency outside allowed range*/
	OSCINIT_NOT_SET = 0x23, /**<OSCINIT/OSCINIT2 did not set within allowed time*/
	RTC_FREQ_OOR	= 0x24, /**<RTC frequency outside allowed range*/
	RTC_NOT_INIT	= 0x25  /**<RTC oscillator did not initialize in time*/
}OSC_ERROR;

/*!IRC and FLL error codes*/
typedef enum{
	S_IRC_OOR		= 0x31, /**<Slow IRC is outside allowed range*/
	F_IRC_OOR		= 0x32, /**<Fast IRC is outside allowed range*/
	FLL_BLW_MIN_0	= 0x33, /**<FLL frequency is below minimum value for range 0*/
	FLL_ABV_MAX_0	= 0x34, /**<FLL frequency is above maximum value for range 0*/
	FLL_BLW_MIN_1	= 0x35, /**<FLL frequency is below minimum value for range 1*/
	FLL_ABV_MAX_1	= 0x36, /**<FLL frequency is above maximum value for range 1*/
	FLL_BLW_MIN_2	= 0x37, /**<FLL frequency is below minimum value for range 2*/
	FLL_ABV_MAX_2	= 0x38, /**<FLL frequency is above maximum value for range 2*/
	FLL_BLW_MIN_3	= 0x39, /**<FLL frequency is below minimum value for range 3*/
	FLL_ABV_MAX_3	= 0x3A, /**<FLL frequency is above maximum value for range 3*/
	FLL_DRS_ERROR	= 0x3B  /**<FLL DRS range is greater than 1*/
}IRC_FLL_ERROR;

/*!PLL error codes*/
typedef enum{
	PRDIV_OOR		= 0x41, /**<PRDIV outside allowed range*/
	VDIV_OOR		= 0x42, /**<VDIV outside allowed range*/
	PLL_RCLK_OOR	= 0x43, /**<PLL reference clock frequency is outside allowed range*/
	LOCK_2_NOT_SET	= 0x44, /**<LOCK or LOCK2 bit did not set */
	PLL_OUTPUT_OOR	= 0x45  /**<PLL output frequency is outside allowed range (NEED TO ADD THIS CHECK TO fbe_pbe and blpe_pbe) only in fei-pee at this time*/
}PLL_ERROR;

uint32_t pll_init(uint32_t crystal_val, uint8_t hgo_val, uint8_t erefs_val, int8_t prdiv_val, int8_t vdiv_val, uint8_t mcgout_select);

/********************************************************************/
#endif /* __MCG_H__ */
