/*! @addtogroup LIBS Input-Output Library
 *  @{
 *  @ingroup 	LIBRARIES
 *  @file    	io.h
 *	@brief		Input-Output Header file.
 * 
 * @}
 */
#ifndef IO_H_
#define IO_H_

#include "freedom.h"
#include "types.h"
#include "uart.h"
#include "lcd.h"
#include <stdarg.h>
#include <string.h>

typedef struct
{
    int dest;
    void (*func)(uint8_t);
    char *loc;
} PRINTK_INFO;

#define DEST_CONSOLE    (1)
#define DEST_STRING     (2)

#define FLAGS_MINUS     (0x01)
#define FLAGS_PLUS      (0x02)
#define FLAGS_SPACE     (0x04)
#define FLAGS_ZERO      (0x08)
#define FLAGS_POUND     (0x10)

#define IS_FLAG_MINUS(a)    (a & FLAGS_MINUS)
#define IS_FLAG_PLUS(a)     (a & FLAGS_PLUS)
#define IS_FLAG_SPACE(a)    (a & FLAGS_SPACE)
#define IS_FLAG_ZERO(a)     (a & FLAGS_ZERO)
#define IS_FLAG_POUND(a)    (a & FLAGS_POUND)

#define LENMOD_h        (0x01)
#define LENMOD_l        (0x02)
#define LENMOD_L        (0x04)

#define IS_LENMOD_h(a)  (a & LENMOD_h)
#define IS_LENMOD_l(a)  (a & LENMOD_l)
#define IS_LENMOD_L(a)  (a & LENMOD_L)

#define FMT_d   (0x0001)
#define FMT_o   (0x0002)
#define FMT_x   (0x0004)
#define FMT_X   (0x0008)
#define FMT_u   (0x0010)
#define FMT_c   (0x0020)
#define FMT_s   (0x0040)
#define FMT_p   (0x0080)
#define FMT_n   (0x0100)

#define IS_FMT_d(a)     (a & FMT_d)
#define IS_FMT_o(a)     (a & FMT_o)
#define IS_FMT_x(a)     (a & FMT_x)
#define IS_FMT_X(a)     (a & FMT_X)
#define IS_FMT_u(a)     (a & FMT_u)
#define IS_FMT_c(a)     (a & FMT_c)
#define IS_FMT_s(a)     (a & FMT_s)
#define IS_FMT_p(a)     (a & FMT_p)
#define IS_FMT_n(a)     (a & FMT_n)

uint8_t get_char (void);
void put_char (uint8_t ch);
void put_serial(char *p);
void put_lcd(const char *ptr_str);
int printk (PRINTK_INFO *info, const char *fmt, va_list ap);
int printf_serial (const char *fmt, ...);
int printf_lcd (const char *fmt, ...);
int sprintf (char *s, const char *fmt, ...);

#endif /* IO_H_ */
