/*! @file	pin_assig.h
 *  @brief Pin Assignment Header File.
 */
#ifndef PIN_ASSIG_H_
#define PIN_ASSIG_H_

#include "derivative.h"
#include "types.h"

#define GPIOn_BASE  0x400FF000u
#define PORTn_BASE  0x40049000u

/* PORT - Peripheral instance base addresses */
/*! Peripheral PORTA base pointer */
#define PTA                           ((PORT_MemMapPtr)0x40049000u)
/*! Peripheral PORTB base pointer */
#define PTB                           ((PORT_MemMapPtr)0x4004A000u)
/*! Peripheral PORTC base pointer */
#define PTC                           ((PORT_MemMapPtr)0x4004B000u)
/*! Peripheral PORTD base pointer */
#define PTD                           ((PORT_MemMapPtr)0x4004C000u)
/*! Peripheral PORTE base pointer */
#define PTE                           ((PORT_MemMapPtr)0x4004D000u)


/*!Pin Initialization Structure */
struct pin_struct
{
	PORT_MemMapPtr      port; 	/*! Peripheral base pointer*/ 
	uint8_t             pin;	/*! Pin Signal*/
	uint8_t			   	irq;	/*! Interrupt Configuration*/
	uint8_t            	mux;	/*! Pin Mux Control*/
	uint8_t		    	pe;		/*! Pull Enable*/
	uint8_t				ds;		/*! Drive Strength Enable*/
	uint8_t		    	mode;	/*! Port Data Direction*/
	uint8_t				value;	/*! Initial value in case of GPIO*/
};

typedef struct pin_struct pin_struct_t;


/*! Interrupt Configuration*/
typedef enum
{
    DISABLED = 0, 	/*! Interrupt/DMA request disabled */
    DMA_RIS,		/*! DMA request on rising edge */
    DMA_FALL,		/*! DMA request on falling edge */
    DMA_BOTH,		/*! DMA request on either edge */
    INT_ZERO,		/*! Interrupt when logic zero */
    INT_RIS,		/*! Interrupt on rising edge */
    INT_FALL,		/*! Interrupt on falling edge */
    INT_BOTH,		/*! Interrupt on either edge */
    INT_ONE			/*! Interrupt when logic one */
}interrupt_configuration_t;

/*! Pin Mux Control*/
typedef enum
{
	ALT0 = 0, 	/*! Pin disabled (analog) */
	ALT1,		/*! Alternative 1 (GPIO) */
	ALT2,		/*! Alternative 2 (chip-specific) */
	ALT3,		/*! Alternative 3 (chip-specific) */
	ALT4,		/*! Alternative 4 (chip-specific) */
	ALT5,		/*! Alternative 5 (chip-specific) */
	ALT6,		/*! Alternative 6 (chip-specific) */
	ALT7		/*! Alternative 7 (chip-specific) */
}alt_funcionality_t;

#define PIN_SHIFT(x)	1 << x

#define	INPUT	0
#define OUTPUT	1

#define FALSE	(0)
#define	TRUE	(1)
#define NULL	(0)

/* Function Prototypes */
void port_init(const pin_struct_t pin_def[]);

extern const pin_struct_t pin_definitions[];

#endif /* PIN_ASSIG_H_ */
